using System;

using Syntriax.Engine.Core;

using YamlDotNet.Core;
using YamlDotNet.Core.Events;
using YamlDotNet.Serialization;

namespace Syntriax.Engine.Serializers.Yaml;

public class QuaternionConverter : EngineTypeYamlSerializerBase<Quaternion>
{
    private static readonly int SUBSTRING_START_LENGTH = nameof(Quaternion).Length + 1;

    public override Quaternion Read(IParser parser, Type type, ObjectDeserializer rootDeserializer)
    {
        string value = parser.Consume<Scalar>().Value;
        string insideParenthesis = value[SUBSTRING_START_LENGTH..^1];
        string[] values = insideParenthesis.Split(", ");
        return new Quaternion(float.Parse(values[0]), float.Parse(values[1]), float.Parse(values[2]), float.Parse(values[3]));
    }

    public override void WriteYaml(IEmitter emitter, object? value, Type type, ObjectSerializer serializer)
    {
        Quaternion quaternion = (Quaternion)value!;
        emitter.Emit(new Scalar($"{nameof(Quaternion)}({quaternion.X}, {quaternion.Y}, {quaternion.Z}, {quaternion.W})"));
    }
}
